@@ -1,8 +1,8 @@
 /***************************************************************************
  *            verve-plugin.c
  *
- *  Copyright  2006-2007  Jannis Pohlmann
- *  jannis@xfce.org
+ *  Copyright © 2006-2007 Jannis Pohlmann <jannis@xfce.org>
+ *  Copyright © 2015 Isaac Schemm <isaacschemm@gmail.com>
  ****************************************************************************/
 
 /*
@@ -52,6 +52,7 @@ typedef struct
 
   /* User interface */
   GtkWidget        *event_box;
+  GtkWidget        *label;
   GtkWidget        *input;
   
   /* Command history */
@@ -67,6 +68,10 @@ typedef struct
   /* Properties */ 
   gint              size;
   gint              history_length;
+  VerveLaunchParams launch_params;
+
+  /* Default GTK style - to restore after blinking upon verve-focus */
+  GtkStyle         *default_style;
 
 #ifdef HAVE_DBUS
   VerveDBusService *dbus_service;
@@ -132,9 +137,9 @@ verve_plugin_focus_timeout (VervePlugin *verve)
   if (gdk_color_equal (&style->base[GTK_STATE_NORMAL], &style->base[GTK_STATE_SELECTED]))
     {
       /* Make it look normal again */
-      gtk_widget_modify_base (verve->input, GTK_STATE_NORMAL, &style->base[GTK_STATE_NORMAL]);
-      gtk_widget_modify_bg (verve->input, GTK_STATE_NORMAL, &style->bg[GTK_STATE_NORMAL]);
-      gtk_widget_modify_text (verve->input, GTK_STATE_NORMAL, &style->text[GTK_STATE_NORMAL]);
+      gtk_widget_modify_base (verve->input, GTK_STATE_NORMAL, &verve->default_style->base[GTK_STATE_NORMAL]);
+      gtk_widget_modify_bg (verve->input, GTK_STATE_NORMAL, &verve->default_style->bg[GTK_STATE_NORMAL]);
+      gtk_widget_modify_text (verve->input, GTK_STATE_NORMAL, &verve->default_style->text[GTK_STATE_NORMAL]);
     }
   else
     {
@@ -164,13 +169,10 @@ verve_plugin_focus_timeout_reset (VervePlugin *verve)
       verve->focus_timeout = 0;
     }
   
-  /* Determine current entry style */
-  style = gtk_widget_get_style (verve->input);
-  
   /* Reset entry background */
-  gtk_widget_modify_base (verve->input, GTK_STATE_NORMAL, &style->base[GTK_STATE_NORMAL]);
-  gtk_widget_modify_bg (verve->input, GTK_STATE_NORMAL, &style->bg[GTK_STATE_NORMAL]);
-  gtk_widget_modify_text (verve->input, GTK_STATE_NORMAL, &style->text[GTK_STATE_NORMAL]);
+  gtk_widget_modify_base (verve->input, GTK_STATE_NORMAL, &verve->default_style->base[GTK_STATE_NORMAL]);
+  gtk_widget_modify_bg (verve->input, GTK_STATE_NORMAL, &verve->default_style->bg[GTK_STATE_NORMAL]);
+  gtk_widget_modify_text (verve->input, GTK_STATE_NORMAL, &verve->default_style->text[GTK_STATE_NORMAL]);
 }
 
 
@@ -209,7 +211,7 @@ verve_plugin_buttonpress_cb (GtkWidget *entry,
     verve_plugin_focus_timeout_reset (verve);
 
   /* Grab entry focus if possible */
-  if (event->button != 3 && toplevel && toplevel->window)
+  if (event->button != 3 && toplevel && toplevel->window && !GTK_WIDGET_HAS_FOCUS(entry))
     xfce_panel_plugin_focus_widget (verve->plugin, entry);
 
   return FALSE;
@@ -233,7 +235,7 @@ verve_plugin_grab_focus (VerveDBusService *dbus_service,
   if (toplevel && toplevel->window)
     {
       /* Unhide the panel */
-      xfce_panel_plugin_set_panel_hidden (verve->plugin, FALSE);
+      xfce_panel_plugin_block_autohide (verve->plugin, TRUE);
 
       /* Focus the command entry */
       xfce_panel_plugin_focus_widget (verve->plugin, verve->input);
@@ -376,10 +378,10 @@ verve_plugin_keypress_cb (GtkWidget   *entry,
           terminal = FALSE;
         
         /* Try executing the command */
-        if (G_LIKELY (verve_execute (command, terminal)))
+        if (G_LIKELY (verve_execute (command, terminal, verve->launch_params)))
           {
             /* Hide the panel again */
-            xfce_panel_plugin_set_panel_hidden (verve->plugin, TRUE);
+            xfce_panel_plugin_block_autohide (verve->plugin, FALSE);
 
             /* Do not add command to history if it is the same as the one before */
             if (verve_history_is_empty () || g_utf8_collate (verve_history_get_last_command (), command) != 0)
@@ -407,7 +409,7 @@ verve_plugin_keypress_cb (GtkWidget   *entry,
             gchar *msg = g_strconcat (_("Could not execute command:"), " ", command, NULL);
 
             /* Display error message dialog */
-            xfce_dialog_show_error (NULL, NULL, msg);
+            xfce_dialog_show_error (NULL, NULL, "%s", msg);
 
             /* Free message */
             g_free (msg);
@@ -517,6 +519,13 @@ verve_plugin_new (XfcePanelPlugin *plugin)
   verve->n_complete = 0;
   verve->size = 20;
   verve->history_length = 25;
+  verve->launch_params.use_bang = FALSE;
+  verve->launch_params.use_backslash = FALSE;
+  verve->launch_params.use_smartbookmark = FALSE;
+  verve->launch_params.smartbookmark_url = g_strdup("");
+
+  /* Initialize label */
+  verve->label = gtk_label_new ("");
 
   /* Connect to load-binaries signal of environment */
   g_signal_connect (G_OBJECT (verve_env_get()), "load-binaries", G_CALLBACK (verve_plugin_load_completion), verve);
@@ -528,16 +537,30 @@ verve_plugin_new (XfcePanelPlugin *plugin)
   verve->event_box = gtk_event_box_new ();
   gtk_widget_show (verve->event_box);
   
+  /* Create a container for the label and input */
+  GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (verve->event_box), hbox);
+  gtk_widget_show (hbox);
+
+  /* Add the label */
+  gtk_widget_show (verve->label);
+  gtk_container_add (GTK_CONTAINER (hbox), verve->label);
+  
   /* Create the input entry */
   verve->input = gtk_entry_new ();
   gtk_entry_set_width_chars (GTK_ENTRY (verve->input), 20);
   gtk_widget_show (verve->input);
-  gtk_container_add (GTK_CONTAINER (verve->event_box), verve->input);
+  gtk_container_add (GTK_CONTAINER (hbox), verve->input);
 
   /* Handle mouse button and key press events */
   g_signal_connect (verve->input, "key-press-event", G_CALLBACK (verve_plugin_keypress_cb), verve);
   g_signal_connect (verve->input, "button-press-event", G_CALLBACK (verve_plugin_buttonpress_cb), verve);
   g_signal_connect (verve->input, "focus-out-event", G_CALLBACK (verve_plugin_focus_out), verve);
+
+  /* Copy the default GTK style into the plugin structure, for restoring after verve-focus */
+  GtkStyle *style = gtk_widget_get_style (verve->input);
+  verve->default_style = g_new (GtkStyle, 1);
+  memcpy(verve->default_style, style, sizeof (GtkStyle));
   
 #ifdef HAVE_DBUS
   /* Attach the D-BUS service */
@@ -566,7 +589,10 @@ verve_plugin_free (XfcePanelPlugin *plugin,
 
   /* Unload completion */
   g_completion_free (verve->completion);
-  
+
+  /* Unload default GTK style */
+  g_free (verve->default_style);
+
   /* Free plugin data structure */
   g_free (verve);
 
@@ -608,6 +634,21 @@ verve_plugin_size_changed_request (XfcePanelPlugin *plugin,
 
 
 
+static gboolean
+verve_plugin_update_label (XfcePanelPlugin *plugin,
+                           const gchar     *label,
+                           VervePlugin     *verve)
+{
+  g_return_val_if_fail (verve != NULL, FALSE);
+
+  /* Set text in internal label object */
+  gtk_label_set_text(verve->label, label);
+
+  return TRUE;
+}
+
+
+
 static gboolean
 verve_plugin_update_history_length (XfcePanelPlugin *plugin,
                                     gint             history_length,
@@ -626,6 +667,22 @@ verve_plugin_update_history_length (XfcePanelPlugin *plugin,
 
 
 
+static gboolean
+verve_plugin_update_smartbookmark_url (XfcePanelPlugin *plugin,
+                                       const gchar     *url,
+                                       VervePlugin     *verve)
+{
+  g_return_val_if_fail (verve != NULL, FALSE);
+
+  /* Set internal search engine URL variable */
+  g_free (verve->launch_params.smartbookmark_url);
+  verve->launch_params.smartbookmark_url = g_strdup(url);
+
+  return TRUE;
+}
+
+
+
 static void
 verve_plugin_read_rc_file (XfcePanelPlugin *plugin, 
                            VervePlugin *verve)
@@ -636,9 +693,25 @@ verve_plugin_read_rc_file (XfcePanelPlugin *plugin,
   /* Default size */
   gint    size = 20;
 
+  /* Default label */
+  gchar  *label = "";
+
   /* Default number of saved history entries */
   gint    history_length = 25;
 
+  /* Default launch parameters */
+  verve->launch_params.use_url = TRUE;
+  verve->launch_params.use_email = TRUE;
+  verve->launch_params.use_dir = TRUE;
+  verve->launch_params.use_wordexp = TRUE;
+  verve->launch_params.use_bang = FALSE;
+  verve->launch_params.use_backslash = FALSE;
+  verve->launch_params.use_smartbookmark = FALSE;
+  verve->launch_params.use_shell = TRUE;
+
+  /* Default search engine URL */
+  gchar *smartbookmark_url = "";
+
   g_return_if_fail (plugin != NULL);
   g_return_if_fail (verve != NULL);
 
@@ -658,14 +731,36 @@ verve_plugin_read_rc_file (XfcePanelPlugin *plugin,
       /* Read size value */
       size = xfce_rc_read_int_entry (rc, "size", size);
 
+      /* Read label text */
+      label = xfce_rc_read_entry (rc, "label", label);
+
       /* Read number of saved history entries */
       history_length = xfce_rc_read_int_entry (rc, "history-length", history_length);
+
+      /* Read launch parameters */
+      verve->launch_params.use_url = xfce_rc_read_bool_entry (rc, "use-url", verve->launch_params.use_url);
+      verve->launch_params.use_email = xfce_rc_read_bool_entry (rc, "use-email", verve->launch_params.use_email);
+      verve->launch_params.use_dir = xfce_rc_read_bool_entry (rc, "use-dir", verve->launch_params.use_dir);
+      verve->launch_params.use_wordexp = xfce_rc_read_bool_entry (rc, "use-wordexp", verve->launch_params.use_wordexp);
+      verve->launch_params.use_bang = xfce_rc_read_bool_entry (rc, "use-bang", verve->launch_params.use_bang);
+      verve->launch_params.use_backslash = xfce_rc_read_bool_entry (rc, "use-backslash", verve->launch_params.use_backslash);
+      verve->launch_params.use_smartbookmark = xfce_rc_read_bool_entry (rc, "use-smartbookmark", verve->launch_params.use_smartbookmark);
+      verve->launch_params.use_shell = xfce_rc_read_bool_entry (rc, "use-shell", verve->launch_params.use_shell);
+
+      /* Read smartbookmark URL */
+      smartbookmark_url = xfce_rc_read_entry (rc, "smartbookmark-url", smartbookmark_url);
     
       /* Update plugin size */
       verve_plugin_update_size (NULL, size, verve);
+    
+      /* Update label */
+      verve_plugin_update_label (NULL, label, verve);
 
       /* Update history length */
       verve_plugin_update_history_length (NULL, history_length, verve);
+
+      /* Update smartbookmark URL */
+      verve_plugin_update_smartbookmark_url (NULL, smartbookmark_url, verve);
       
       /* Close handle */
       xfce_rc_close (rc);
@@ -702,8 +797,24 @@ verve_plugin_write_rc_file (XfcePanelPlugin *plugin,
       /* Write size value */
       xfce_rc_write_int_entry (rc, "size", verve->size);
 
+      /* Write label value */
+      xfce_rc_write_entry (rc, "label", gtk_label_get_text(verve->label));
+
       /* Write number of saved history entries */
       xfce_rc_write_int_entry (rc, "history-length", verve->history_length);
+
+      /* Write launch param settings */
+      xfce_rc_write_bool_entry (rc, "use-url", verve->launch_params.use_url);
+      xfce_rc_write_bool_entry (rc, "use-email", verve->launch_params.use_email);
+      xfce_rc_write_bool_entry (rc, "use-dir", verve->launch_params.use_dir);
+      xfce_rc_write_bool_entry (rc, "use-wordexp", verve->launch_params.use_wordexp);
+      xfce_rc_write_bool_entry (rc, "use-bang", verve->launch_params.use_bang);
+      xfce_rc_write_bool_entry (rc, "use-backslash", verve->launch_params.use_backslash);
+      xfce_rc_write_bool_entry (rc, "use-smartbookmark", verve->launch_params.use_smartbookmark);
+      xfce_rc_write_bool_entry (rc, "use-shell", verve->launch_params.use_shell);
+
+      /* Write smartbookmark URL */
+      xfce_rc_write_entry (rc, "smartbookmark-url", verve->launch_params.smartbookmark_url);
     
       /* Close handle */
       xfce_rc_close (rc);
@@ -727,6 +838,21 @@ verve_plugin_size_changed (GtkSpinButton *spin,
 
 
 
+static void
+verve_plugin_label_changed (GtkEntry *box, 
+                            VervePlugin *verve)
+{
+  g_return_if_fail (verve != NULL);
+
+  /* Get the entered URL */
+  const gchar *label = gtk_entry_get_text (box);
+
+  /* Update search engine ID */
+  verve_plugin_update_label (NULL, label, verve);
+}
+
+
+
 static void
 verve_plugin_history_length_changed (GtkSpinButton *spin,
                                      VervePlugin   *verve)
@@ -739,6 +865,101 @@ verve_plugin_history_length_changed (GtkSpinButton *spin,
 
 
 
+static void
+verve_plugin_use_url_changed (GtkToggleButton *button, 
+                              VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_url = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_use_email_changed (GtkToggleButton *button, 
+                                VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_email = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_use_dir_changed (GtkToggleButton *button, 
+                              VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_dir = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_use_wordexp_changed (GtkToggleButton *button, 
+                                  VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_wordexp = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_use_bang_changed (GtkToggleButton *button, 
+                               VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_bang = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_use_backslash_changed (GtkToggleButton *button, 
+                                    VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_backslash = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_use_smartbookmark_changed (GtkToggleButton *button, 
+                                        VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_smartbookmark = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_use_shell_changed (GtkToggleButton *button, 
+                                VervePlugin     *verve)
+{
+  g_return_if_fail (verve != NULL);
+  verve->launch_params.use_shell = gtk_toggle_button_get_active (button);
+}
+
+
+
+static void
+verve_plugin_smartbookmark_url_changed (GtkEntry    *box, 
+                                        VervePlugin *verve)
+{
+  g_return_if_fail (verve != NULL);
+
+  /* Get the entered URL */
+  const gchar *smartbookmark_url = gtk_entry_get_text(box);
+
+  /* Update search engine ID */
+  verve_plugin_update_smartbookmark_url (NULL, smartbookmark_url, verve);
+}
+
+
+
 static void
 verve_plugin_response (GtkWidget *dialog, 
                        int response, 
@@ -767,16 +988,39 @@ verve_plugin_properties (XfcePanelPlugin *plugin,
                          VervePlugin *verve)
 {
   GtkWidget *dialog;
+  GtkWidget *notebook;
+  GtkWidget *notebook_tab_label_current;
+  
+  GtkWidget *general_vbox;
   GtkWidget *frame;
   GtkWidget *bin1;
   GtkWidget *bin2;
   GtkWidget *hbox;
+  GtkWidget *vbox;
   GtkWidget *size_label;
   GtkWidget *size_spin;
+  GtkWidget *label_label;
+  GtkWidget *label_box;
   GtkWidget *history_length_label;
   GtkWidget *history_length_spin;
   GtkObject *adjustment;
 
+  GtkWidget *bin3;
+  GtkWidget *alignment;
+  GtkWidget *command_types_vbox;
+  GtkWidget *command_types_label1;
+  GtkWidget *command_types_label2;
+  GtkWidget *command_type_url;
+  GtkWidget *command_type_email;
+  GtkWidget *command_type_dir;
+  GtkWidget *command_type_use_wordexp;
+  GtkWidget *command_type_bang;
+  GtkWidget *command_type_backslash;
+  GtkWidget *command_type_smartbookmark;
+  GtkWidget *engine_box;
+  GtkWidget *command_type_executable;
+  GtkWidget *command_type_use_shell;
+
   g_return_if_fail (plugin != NULL);
   g_return_if_fail (verve != NULL);
 
@@ -800,16 +1044,31 @@ verve_plugin_properties (XfcePanelPlugin *plugin,
   gtk_window_set_icon_name (GTK_WINDOW (dialog), "utilities-terminal");
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 2);
+  
+  /* Notebook setup */
+  notebook = gtk_notebook_new ();
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), notebook, TRUE, TRUE, 0);
+  gtk_widget_show (notebook);
+  
+  general_vbox = gtk_vbox_new (FALSE, 8);
+  notebook_tab_label_current = gtk_label_new (_("General"));
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), general_vbox, notebook_tab_label_current);
+  gtk_widget_show(general_vbox);
 
   /* Frame for appearance settings */
   frame = xfce_gtk_frame_box_new (_("Appearance"), &bin1);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), frame, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (general_vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
+  /* Appearance settings vertical container */
+  vbox = gtk_vbox_new (FALSE, 8);
+  gtk_container_add (GTK_CONTAINER(bin1), vbox);
+  gtk_widget_show(vbox);
+
   /* Plugin size container */
   hbox = gtk_hbox_new (FALSE, 8);
-  gtk_container_add (GTK_CONTAINER (bin1), hbox);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   /* Plugin size label */
@@ -818,7 +1077,7 @@ verve_plugin_properties (XfcePanelPlugin *plugin,
   gtk_widget_show (size_label);
 
   /* Plugin size adjustment */
-  adjustment = gtk_adjustment_new (verve->size, 5, 100, 1, 5, 0);
+  adjustment = gtk_adjustment_new (verve->size, 5, 300, 1, 5, 0);
 
   /* Plugin size spin button */
   size_spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
@@ -832,15 +1091,42 @@ verve_plugin_properties (XfcePanelPlugin *plugin,
   /* Be notified when the user requests a different plugin size */
   g_signal_connect (size_spin, "value-changed", G_CALLBACK (verve_plugin_size_changed), verve);
 
+  /* Plugin label container */
+  hbox = gtk_hbox_new (FALSE, 8);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+  gtk_widget_show (hbox);
+
+  /* Plugin label label */
+  label_label = gtk_label_new (_("Label:"));
+  gtk_box_pack_start (GTK_BOX (hbox), label_label, FALSE, TRUE, 0);
+  gtk_widget_show (label_label);
+
+  /* Plugin label entry field */
+  label_box = gtk_entry_new();
+
+  /* Set text to current plugin label */
+  gtk_entry_set_text(GTK_ENTRY (label_box), gtk_label_get_text(GTK_LABEL (verve->label)));
+  gtk_widget_add_mnemonic_label (label_box, label_label);
+  gtk_box_pack_start (GTK_BOX (hbox), label_box, FALSE, TRUE, 0);
+  gtk_widget_show (label_box);
+
+  /* Be notified when the user requests a different label setting */
+  g_signal_connect (label_box, "changed", G_CALLBACK (verve_plugin_label_changed), verve);
+
   /* Frame for behaviour settings */
-  frame = xfce_gtk_frame_box_new (_("Behaviour"), &bin2);
+  frame = xfce_gtk_frame_box_new (_("History"), &bin2);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), frame, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (general_vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
+  
+  /* Behaviour settings vertical container */
+  vbox = gtk_vbox_new (FALSE, 8);
+  gtk_container_add (GTK_CONTAINER(bin2), vbox);
+  gtk_widget_show(vbox);
 
   /* Plugin history length container */
   hbox = gtk_hbox_new (FALSE, 8);
-  gtk_container_add (GTK_CONTAINER (bin2), hbox);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   /* History length label */
@@ -862,8 +1148,132 @@ verve_plugin_properties (XfcePanelPlugin *plugin,
 
   /* Be notified when the user requests a different history length */
   g_signal_connect (history_length_spin, "value-changed", G_CALLBACK (verve_plugin_history_length_changed), verve);
+  
+  /* Second tab */
+  frame = xfce_gtk_frame_box_new (_("Behaviour"), &bin3);
+  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
+  gtk_widget_show (frame);
+  
+  notebook_tab_label_current = gtk_label_new (_("Behaviour"));
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, notebook_tab_label_current);
+  gtk_widget_show (frame);
+  
+  command_types_vbox = gtk_vbox_new (FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (bin3), command_types_vbox);
+  gtk_widget_show (command_types_vbox);
+
+  /* Pattern types frame label */
+  command_types_label1 = gtk_label_new(_("Enable support for:"));
+  gtk_misc_set_alignment (GTK_MISC (command_types_label1), 0, 0.5);
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_types_label1, FALSE, TRUE, 8);
+  gtk_widget_show (command_types_label1);
+  
+  /* Command type: URL */
+  command_type_url = gtk_check_button_new_with_label(_("URLs (http/https/ftp/ftps)"));
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_type_url, FALSE, TRUE, 0);
+  gtk_widget_show (command_type_url);
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_url), verve->launch_params.use_url);
+  g_signal_connect (command_type_url, "toggled", G_CALLBACK (verve_plugin_use_url_changed), verve);
+  
+  /* Command type: email address */
+  command_type_email = gtk_check_button_new_with_label(_("Email addresses"));
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_type_email, FALSE, TRUE, 0);
+  gtk_widget_show (command_type_email);
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_email), verve->launch_params.use_email);
+  g_signal_connect (command_type_email, "toggled", G_CALLBACK (verve_plugin_use_email_changed), verve);
+  
+  /* Command type: directory path */
+  command_type_dir = gtk_check_button_new_with_label(_("Directory paths"));
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_type_dir, FALSE, TRUE, 0);
+  gtk_widget_show (command_type_dir);
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_dir), verve->launch_params.use_dir);
+  g_signal_connect (command_type_dir, "toggled", G_CALLBACK (verve_plugin_use_dir_changed), verve);
+
+  /* wordexp checkbox */
+  alignment = gtk_alignment_new (1, 1, 1, 1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 24, 0);
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), alignment, FALSE, TRUE, 0);
+  gtk_widget_show (alignment);
+  
+  command_type_use_wordexp = gtk_check_button_new_with_label(_("Expand variables with wordexp"));
+  gtk_container_add (GTK_CONTAINER (alignment), command_type_use_wordexp);
+#ifdef HAVE_WORDEXP
+  gtk_widget_show (command_type_use_wordexp);
+#endif
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_use_wordexp), verve->launch_params.use_wordexp);
+  g_signal_connect (command_type_use_wordexp, "toggled", G_CALLBACK (verve_plugin_use_wordexp_changed), verve);
+  
+  /* Command type: !bang */
+  command_type_bang = gtk_check_button_new_with_label(_("DuckDuckGo queries (starting with !)"));
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_type_bang, FALSE, TRUE, 0);
+  gtk_widget_show (command_type_bang);
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_bang), verve->launch_params.use_bang);
+  g_signal_connect (command_type_bang, "toggled", G_CALLBACK (verve_plugin_use_bang_changed), verve);
+  
+  /* Command type: I'm feeling ducky (\) */
+  command_type_backslash = gtk_check_button_new_with_label(_("DuckDuckGo queries (starting with \\)"));
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_type_backslash, FALSE, TRUE, 0);
+  gtk_widget_show (command_type_backslash);
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_backslash), verve->launch_params.use_backslash);
+  g_signal_connect (command_type_backslash, "toggled", G_CALLBACK (verve_plugin_use_backslash_changed), verve);
+  
+  /* Fallback if the above don't match */
+  command_types_label2 = gtk_label_new(_("If the above patterns don't match:"));
+  gtk_misc_set_alignment (GTK_MISC (command_types_label2), 0, 0.5);
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_types_label2, FALSE, TRUE, 8);
+  gtk_widget_show (command_types_label2);
+  
+  /* Smart bookmark radio button */
+  command_type_smartbookmark = gtk_radio_button_new_with_label(NULL, _("Use smart bookmark URL"));
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_type_smartbookmark, FALSE, TRUE, 0);
+  gtk_widget_show (command_type_smartbookmark);
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_smartbookmark), verve->launch_params.use_smartbookmark);
+  g_signal_connect (command_type_smartbookmark, "toggled", G_CALLBACK (verve_plugin_use_smartbookmark_changed), verve);
+
+  /* Smart bookmark entry box */
+  engine_box = gtk_entry_new();
+  alignment = gtk_alignment_new (1, 1, 1, 1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 8, 8, 24, 0);
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), alignment, FALSE, TRUE, 0);
+  gtk_widget_show (alignment);
+  gtk_container_add (GTK_CONTAINER (alignment), engine_box);
+  gtk_widget_show (engine_box);
+
+  /* Set text to search engine URL */
+  gtk_entry_set_text(GTK_ENTRY (engine_box), verve->launch_params.smartbookmark_url);
+
+  /* Be notified when the user requests a different smartbookmark url setting */
+  g_signal_connect (engine_box, "changed", G_CALLBACK (verve_plugin_smartbookmark_url_changed), verve);
+  
+  /* Executable command radio button (smart bookmark off) */
+  command_type_executable = gtk_radio_button_new_with_label(gtk_radio_button_get_group (GTK_RADIO_BUTTON (command_type_smartbookmark)), _("Run as executable command"));
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), command_type_executable, FALSE, TRUE, 0);
+  gtk_widget_show (command_type_executable);
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_executable), !verve->launch_params.use_smartbookmark);
+
+  /* Use shell checkbox */
+  alignment = gtk_alignment_new (1, 1, 1, 1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 24, 0);
+  gtk_box_pack_start (GTK_BOX (command_types_vbox), alignment, FALSE, TRUE, 0);
+  gtk_widget_show (alignment);
+  
+  command_type_use_shell = gtk_check_button_new_with_label(_("Run command with $SHELL -i -c\n(enables alias and variable expansion)"));
+  gtk_container_add (GTK_CONTAINER (alignment), command_type_use_shell);
+  gtk_widget_show (command_type_use_shell);
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (command_type_use_shell), verve->launch_params.use_shell);
+  g_signal_connect (command_type_use_shell, "toggled", G_CALLBACK (verve_plugin_use_shell_changed), verve);
 
   /* Show properties dialog */
+  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
   gtk_widget_show (dialog);
 }
  
