@@ -1,8 +1,8 @@
 /***************************************************************************
  *            verve.c
  *
- *  Copyright  2006-2007  Jannis Pohlmann
- *  jannis@xfce.org
+ *  Copyright © 2006-2007 Jannis Pohlmann <jannis@xfce.org>
+ *  Copyright © 2015 Isaac Schemm <isaacschemm@gmail.com>
  ****************************************************************************/
 
 /*
@@ -20,11 +20,17 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
- 
+
+#include "config.h"
+
 #include <pcre.h>
 
 #include <glib-object.h>
 
+#ifdef HAVE_WORDEXP
+#include <wordexp.h>
+#endif
+
 #include <libxfce4ui/libxfce4ui.h>
 
 #include "verve.h"
@@ -35,7 +41,7 @@
 
 static gboolean verve_is_url       (const gchar *str);
 static gboolean verve_is_email     (const gchar *str);
-static gboolean verve_is_directory (const gchar *str);
+static gchar *verve_is_directory   (const gchar *str, gboolean use_wordexp);
 
 
 
@@ -50,8 +56,6 @@ static gboolean verve_is_directory (const gchar *str);
                     "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?/?$"
 #define MATCH_EMAIL "^(mailto:)?[a-z0-9][a-z0-9.-]*@[a-z0-9][a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+$"
 
-
-
 /*********************************************************************
  *
  * Initialize/shutdown Verve
@@ -83,6 +87,26 @@ verve_shutdown (void)
 
 
 
+void
+verve_command_callback (GPid pid,
+                        gint status,
+                        gpointer data)
+{
+  status >>= 8;
+  if (status == 126 || status == 127)
+  {
+    xfce_dialog_show_error (NULL, NULL, _("Could not execute command (exit status %d)"), status);
+  }
+  g_spawn_close_pid (pid);
+}
+
+
+
+static void verve_setsid (gpointer p)
+{
+    setsid();
+}
+
 /*********************************************************************
  *
  * Verve command line execution function
@@ -97,6 +121,7 @@ gboolean verve_spawn_command_line (const gchar *cmdline)
   gint         argc;
   gchar      **argv;
   gboolean     success;
+  GPid         child_pid;
   GError      *error = NULL;
   const gchar *home_dir;
   GSpawnFlags  flags;
@@ -118,9 +143,10 @@ gboolean verve_spawn_command_line (const gchar *cmdline)
   flags = G_SPAWN_STDOUT_TO_DEV_NULL;
   flags |= G_SPAWN_STDERR_TO_DEV_NULL;
   flags |= G_SPAWN_SEARCH_PATH;
+  flags |= G_SPAWN_DO_NOT_REAP_CHILD;
   
   /* Spawn subprocess */
-  success = g_spawn_async (home_dir, argv, NULL, flags, NULL, NULL, NULL, &error);
+  success = g_spawn_async (home_dir, argv, NULL, flags, verve_setsid, NULL, &child_pid, &error);
 
   /* Return false if subprocess could not be spawned */
   if (G_UNLIKELY (error != NULL)) 
@@ -131,7 +157,9 @@ gboolean verve_spawn_command_line (const gchar *cmdline)
 
   /* Free command line arguments */
   g_strfreev (argv);
-  
+
+  g_child_watch_add(child_pid, (GChildWatchFunc)verve_command_callback, NULL);
+
   /* Return whether process was spawned successfully */
   return success;
 }
@@ -150,24 +178,66 @@ gboolean verve_spawn_command_line (const gchar *cmdline)
 
 gboolean
 verve_execute (const gchar *input, 
-               gboolean terminal)
+               gboolean terminal,
+               VerveLaunchParams launch_params)
 {
   gchar   *command;
+  gchar   *directory_exp;
   gboolean result = FALSE;
     
   /* Open URLs, eMail addresses and directories using exo-open */
-  if (verve_is_url (input) || verve_is_email (input) || verve_is_directory (input))
+  if ((launch_params.use_email && verve_is_email (input)) || (launch_params.use_url && verve_is_url (input)))
   {
     /* Build exo-open command */
     command = g_strconcat ("exo-open ", input, NULL);
   }
+  else if (launch_params.use_dir && (directory_exp = verve_is_directory (input, launch_params.use_wordexp)))
+  {
+    /* Build exo-open command */
+    command = g_strconcat ("exo-open ", directory_exp, NULL);
+    g_free (directory_exp);
+  }
+  else if ((launch_params.use_bang && input[0] == '!') || (launch_params.use_backslash && input[0] == '\\'))
+  {
+    /* Launch DuckDuckGo */
+    gchar *esc_input = g_uri_escape_string(input, NULL, TRUE);
+    command = g_strconcat ("exo-open https://duckduckgo.com/?q=", esc_input, NULL);
+    g_free(esc_input);
+  }
+  else if (launch_params.use_smartbookmark)
+  {
+    /* Launch user-defined search engine */
+    gchar *esc_input = g_uri_escape_string(input, NULL, TRUE);
+    command = g_strconcat ("exo-open ", launch_params.smartbookmark_url, esc_input, NULL);
+    g_free(esc_input);
+  }
   else
   {
+    if (launch_params.use_shell)
+    {
+      /* Find current shell */
+      char *shell = getenv("SHELL");
+      if (shell == NULL) shell = "/bin/sh";
+      
+      gchar *quoted_input = g_shell_quote (input);
+      command = g_strconcat (shell, " -i -c ", quoted_input, NULL);
+      g_free (quoted_input);
+    }
+    else
+    {
+      command = g_strdup (input);
+    }
+    
     /* Run command using the xfterm4 wrapper if the terminal flag was set */
     if (G_UNLIKELY (terminal))
-      command = g_strconcat ("exo-open --launch TerminalEmulator '", input, "'", NULL);
-    else
-      command = g_strdup (input);
+    {
+      gchar *quoted_command = g_shell_quote (command);
+      
+      g_free (command);
+      command = g_strconcat ("exo-open --launch TerminalEmulator ", quoted_command, NULL);
+      
+      g_free (quoted_command);
+    }
   }
     
   /* Try to execute the exo-open command */
@@ -264,17 +334,43 @@ verve_is_email (const gchar *str)
 
 
 
-gboolean
-verve_is_directory (const gchar *str)
+gchar *
+verve_is_directory (const gchar *str,
+                    gboolean use_wordexp)
 {
-  /* Avoid opening directories with the same name as an existing executable. */
-  if (g_find_program_in_path (str))
-    return FALSE;
+#ifdef HAVE_WORDEXP
+  if (use_wordexp) {
+    wordexp_t w;
+    int result;
+
+    /* Avoid opening directories with the same name as an existing executable. */
+    if (g_find_program_in_path (str))
+      return NULL;
+
+    /* Run wordexp with command substitution turned off */
+    result = wordexp(str, &w, WRDE_NOCMD);
+
+    /* Only use result if it expanded successfully to exactly one "word" and the result is a directory */
+    if (result == 0) {
+      const char* to_return;
+      if (w.we_wordc != 1)
+        to_return = NULL;
+      else if (g_file_test (w.we_wordv[0], G_FILE_TEST_IS_DIR))
+        to_return = g_strdup (w.we_wordv[0]);
+      else
+        to_return = NULL;
+
+      wordfree (&w);
+      if (to_return)
+        return to_return;
+    }
+  }
+#endif
 
   if (g_file_test (str, G_FILE_TEST_IS_DIR))
-    return TRUE;
+    return g_strdup (str);
   else
-    return FALSE;
+    return NULL;
 }
 
 
